@@ -41,10 +41,12 @@ class Zone:
         """Returns amazon change xml for a specific route 53 entry."""
         rtype = 'A'
         words = rvalue.split()
+        zone_id = words[1]
+        dns_name = words[2]
         xmlout = "   <Change>\n" + "    <Action>" + action + "</Action>\n" + "     <ResourceRecordSet>\n" + \
             "      <Name>" + name + "</Name>\n" + "      <Type>" + rtype + "</Type>\n" + \
-            "      <AliasTarget>\n            <HostedZoneId>" + words[1] + "</HostedZoneId>\n" + \
-            "            <DNSName>" + words[2][:-1] + "</DNSName>\n      </AliasTarget>\n" + \
+            "      <AliasTarget>\n            <HostedZoneId>" + zone_id + "</HostedZoneId>\n" + \
+            "            <DNSName>" + dns_name + "</DNSName>\n      </AliasTarget>\n" + \
             "     </ResourceRecordSet>\n" + "   </Change>\n"
         return xmlout
 
@@ -53,7 +55,7 @@ class Zone:
         #Check to see if any entries are a masked route53 alias
         if rtype == 'A':
             for rvalue in values:
-                if rvalue[1:7] == 'Alias ':
+                if rvalue[:6] == 'Alias ':
                     log.info('Interpreting entry as a route53 alias.')
                     xmlout = self._change_alias_xml(action, name, rvalue)
                     if len(values) > 1:
@@ -80,15 +82,10 @@ class Zone:
         """Returns amazon Change xml for deleting an entry."""
         return self._change_xml('DELETE', name, rtype, ttl, values)
 
-    def _update_change_xml(self, name, rtype, ttl, old_values, new_values):
-        """Returns amazon Change xml for updating an entry."""
-        xmlout = self._change_xml('DELETE', name, rtype, ttl, old_values)
-        xmlout += self._change_xml('CREATE', name, rtype, ttl, new_values)
-        return xmlout
-
     def _create_changeset(self, adds, deletes, updates):
         """Create amazon changeset xml for adds and deletes and updates to a zone.
-        Return a list of individual changesets.
+            Return a list of individual changesets.
+            For updates the host, rtype and ttl all must be the same only value changes.
         """
         if len(adds) == 0 and len(deletes) == 0 and len(updates) == 0:
             return None
@@ -110,7 +107,8 @@ class Zone:
         for rrecords_slice in rrecord_groups: 
             xmlout = self.xml_header
             for key in rrecords_slice:
-                xmlout += self._update_change_xml(key[0], key[1], key[2], updates[key][0], updates[key][1])
+                xmlout += self._delete_change_xml(key[0], key[1], key[2], updates[key][0])
+                xmlout += self._add_change_xml(key[0], key[1], key[2], updates[key][1])
             xmlout += self.xml_footer
             changesets.append(xmlout)
 
@@ -197,6 +195,7 @@ class Zone:
             rvalue = rdata.to_text()
             if rtype == 'TXT' and rvalue[1:7] == 'Alias ':
                 log.info('Interpreting TXT entry as a route53 alias.')
+                rvalue = rvalue.strip('"') #Strip quotes
                 if name[:7] == '_alias.':
                     name = name[7:]
                 rtype = 'A'
@@ -288,13 +287,14 @@ class Zone:
             for xml in zone_xml:
                 change = self.conn.change_rrsets(self.id, xml)
     
-    def create_host(self, host, rtype, ttl, value):
+    def create_host(self, fqdn, rtype, ttl, value):
         """Create a host entry in this zone."""
         xmlout = self.xml_header
-        xmlout += self._add_change_xml('.'.join([host, self.zone_name]), rtype, ttl, [value, ])
+        xmlout += self._add_change_xml(fqdn, rtype, ttl, [value, ])
         xmlout += self.xml_footer
+        log.debug(xmlout)
+        log.warn("Adding %s %s %s to %s" % (fqdn, rtype, ttl, value))
         self.conn.change_rrsets(self.id, xmlout)
-        log.warn("Updating %s %s %s to %s" % (host, rtype, ttl, value))
 
     def exists(self):
         """Return true if the self.zone_name exists on AWS, false otherwise."""
@@ -308,16 +308,30 @@ class Zone:
 
         return False
 
-    def get_host(self, host):
-        """Return the (rtype, [values, ]) if the host exists in this domain on AWS, None otherwise."""
-        for rrecord in self.conn.get_all_rrsets(self.id): #Adding name=host is not working in boto.
+    def get_host(self, host, ltype=None, lname=None):
+        """Return the (rtype, [values, ]) if the host exists in this domain on AWS, None otherwise.
+            Host should be the fqdn
+        """
+        rrecords = self.conn.get_all_rrsets(self.id, ltype, lname) #Adding name=host is not working in boto.
+        if host[:-1] == '.':
+            fqdn = host
+        else:
+            fqdn = host + '.'
+        for rrecord in rrecords:
             log.debug(rrecord.name)
-            if host == rrecord.name.split('.', 1)[0]:
-                return (rrecord.type, rrecord.resource_records)
+            if fqdn == rrecord.name:
+                if len(rrecord.resource_records) == 0 and rrecord.alias_hosted_zone_id is not None:
+                    log.debug('Host has an Alias record, representing as a an A record.')
+                    return ('A', \
+                        ['Alias %s %s' %(rrecord.alias_hosted_zone_id, rrecord.alias_dns_name), ], rrecord.ttl)
+                else:
+                    return (rrecord.type, rrecord.resource_records, rrecord.ttl)
+
+        if len(rrecords) == 100: #Amazon only returns 100 at a time so keep going
+            return self.get_host(host, rrecords[-1].type, rrecords[-1].name)
 
         return None
 
-
     def update(self, zone_file, dry_run):
         """Compare existing Resource Records to the given zone file and update if needed.
         Do nothing, report only, if dry_run is true.
@@ -332,10 +346,6 @@ class Zone:
         r53zone = self._to_dnszone()
         log.info("Bind zone from r53.\n" + self._print(r53zone) + "\n")
 
-        if dnszone == r53zone: #This compares SOA and root NS records, which are ignored in actual updates
-            log.warn('Zone %s is up to date.' % self.zone_name)
-            return
-    
         adds, deletes, updates = self._compare(r53zone, dnszone)
         changesets = self._create_changeset(adds, deletes, updates)
         if changesets is None:
@@ -348,13 +358,17 @@ class Zone:
                 log.debug(changeset)
                 change = self.conn.change_rrsets(self.id, changeset)
 
-    def update_host(self, host, rtype, ttl, existing, value):
-        """Updates a individual host entry in this zone."""
+    def update_host(self, fqdn, rtype, ttl, existing, value):
+        """Updates a individual host entry in this zone.
+            Existing is the output of get_host(host)
+        """
         xmlout = self.xml_header
-        xmlout += self._update_change_xml('.'.join([host, self.zone_name]) , rtype, ttl, existing[1], [value, ])
+        xmlout += self._delete_change_xml(fqdn, existing[0], existing[2], existing[1])
+        xmlout += self._add_change_xml(fqdn, rtype, ttl, [value, ])
         xmlout += self.xml_footer
+        log.debug(xmlout)
+        log.warn("Updating %s %s %s to %s" % (fqdn, rtype, ttl, value))
         self.conn.change_rrsets(self.id, xmlout)
-        log.warn("Updating %s %s %s to %s" % (host, rtype, ttl, value))
 
     def remove(self, dry_run):
         """Remove this zone from AWS."""
